﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text.RegularExpressions;

namespace HeuristicLab.Problems.Instances.QAPLIB {
  public class QAPLIBInstanceProvider : IProblemInstanceProvider<IQAPInstance> {
    public string Name {
      get { return "QAPLIB"; }
    }

    public string Description {
      get { return "Quadratic Assignment Problem Library"; }
    }

    public Uri Link {
      get { return new Uri("http://www.seas.upenn.edu/qaplib/"); }
    }

    public IEnumerable<IInstanceDescriptor> GetInstanceDescriptors() {
      var solutions = Assembly.GetExecutingAssembly()
        .GetManifestResourceNames()
        .Where(x => x.EndsWith(".sln"))
        .ToDictionary(x => Path.GetFileNameWithoutExtension(x) + ".dat", x => x);

      return Assembly.GetExecutingAssembly()
          .GetManifestResourceNames()
          .Where(x => x.EndsWith(".dat"))
          .OrderBy(x => x)
          .Select(x => new QAPLIBInstanceDescriptor(GetPrettyName(x), GetDescription(), x, solutions.ContainsKey(x) ? solutions[x] : String.Empty));
    }

    public IQAPInstance GetInstance(IInstanceDescriptor id) {
      var descriptor = (QAPLIBInstanceDescriptor)id;
      var instance = new QAPLIBInstance();
      using (var stream = Assembly.GetExecutingAssembly()
        .GetManifestResourceStream(descriptor.InstanceIdentifier)) {
        QAPLIBParser datParser = new QAPLIBParser();
        datParser.Parse(stream);
        if (datParser.Error != null) throw datParser.Error;
        instance.Distances = datParser.Distances;
        instance.Weights = datParser.Weights;

        instance.Name = id.Name;
        instance.Description = id.Description;

        if (!String.IsNullOrEmpty(descriptor.SolutionIdentifier)) {
          using (Stream solStream = Assembly.GetExecutingAssembly()
            .GetManifestResourceStream(descriptor.SolutionIdentifier)) {
            QAPLIBSolutionParser slnParser = new QAPLIBSolutionParser();
            slnParser.Parse(solStream, true);
            if (slnParser.Error != null) throw slnParser.Error;

            instance.BestKnownAssignment = slnParser.Assignment;
            instance.BestKnownQuality = slnParser.Quality;
          }
        }
      }
      return instance;
    }

    private string GetPrettyName(string instanceIdentifier) {
      return Regex.Match(instanceIdentifier, GetType().Namespace + @"\.Data\.(.*)\.dat").Groups[1].Captures[0].Value;
    }

    private string GetDescription() {
      return "Embedded instance of plugin version " + Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyFileVersionAttribute), true).Cast<AssemblyFileVersionAttribute>().First().Version + ".";
    }
  }
}
